/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005-2010, Red Hat Middleware LLC, and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.test.messaging.core;

import java.util.List;
import java.util.Properties;

import javax.sql.DataSource;
import javax.transaction.TransactionManager;

import org.jboss.messaging.core.impl.JDBCPersistenceManager;
import org.jboss.messaging.core.impl.tx.Transaction;

/**
 * A FakeJDBCPersistenceManager
 *
 * @author <a href="mailto:hgao@redhat.com">Howard Gao</a>
 * 
 * Created Dec 24, 2010 12:09:22 PM
 *
 *
 */
public class FakeJDBCPersistenceManager extends JDBCPersistenceManager
{

   public FakeJDBCPersistenceManager(DataSource ds,
                                     TransactionManager tm,
                                     Properties sqlProperties,
                                     boolean createTablesOnStartup,
                                     boolean usingBatchUpdates,
                                     boolean usingBinaryStream,
                                     boolean usingTrailingByte,
                                     int maxParams,
                                     boolean supportsBlobSelect,
                                     boolean supportsSetNullOnBlobs)
   {
      super(ds,
            tm,
            sqlProperties,
            createTablesOnStartup,
            usingBatchUpdates,
            usingBinaryStream,
            usingTrailingByte,
            maxParams,
            supportsBlobSelect,
            supportsSetNullOnBlobs);
   }

   // Constants -----------------------------------------------------

   // Attributes ----------------------------------------------------
   boolean poisoned1pc;
   // Static --------------------------------------------------------

   // Constructors --------------------------------------------------

   // Public --------------------------------------------------------
   public void handleBeforeCommit1PC(final List refsToAdd, final List refsToRemove, final Transaction tx) throws Exception
   {
      if (poisoned1pc) throw new Exception("Fake exception");
      super.handleBeforeCommit1PC(refsToAdd, refsToRemove, tx);
   }

   public void poisonHandleCommit1PC()
   {
      poisoned1pc = true;
   }
   
   public void restoreHandleCommit1PC()
   {
      poisoned1pc = false;
   }

}
